<?php

namespace SGalinski\SgJobs\Service;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgJobs\Domain\Repository\JobRepository;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Persistence\Generic\QueryResult;

/**
 * Frontend filter service class
 */
class FrontendFilterService {

	/**
	 * @param array $filters
	 * @return array
	 * @throws \InvalidArgumentException
	 */
	public static function getLocations(array $filters = []) {
		$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable(
			'tx_sgjobs_domain_model_company'
		);
		$statement = $queryBuilder->select('city')
			->from('tx_sgjobs_domain_model_company')
			->where(
				$queryBuilder->expr()->eq(
					'country', $queryBuilder->createNamedParameter($filters['country'])
				)
			);

		if ($filters['location'] !== '0') {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'city', $queryBuilder->createNamedParameter($filters['location'])
				)
			);
		}

		if ($filters['company'] !== '0') {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'name', $queryBuilder->createNamedParameter($filters['company'])
				)
			);
		}

		$locations = $statement->execute()->fetchAll();
		$result = [];
		$result[0] = '';
		foreach ($locations as $location) {
			$result[$location['city']] = $location['city'];
		}

		return $result;
	}

	/**
	 * @param array $filters
	 * @return array
	 * @throws \InvalidArgumentException
	 */
	public static function getCompanies(array $filters = []): array {
		$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable(
			'tx_sgjobs_domain_model_company'
		);
		$statement = $queryBuilder->select('name')
			->from('tx_sgjobs_domain_model_company');
		if ($filters['location'] !== '0' && $filters['location'] !== NULL) {
			$queryBuilder->where(
				$queryBuilder->expr()->eq(
					'city', $queryBuilder->createNamedParameter($filters['location'])
				)
			);
		}

		if ($filters['country'] !== '0' && $filters['country'] !== NULL) {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'country', $queryBuilder->createNamedParameter($filters['country'])
				)
			);
		}

		if ($filters['company'] !== '0' && $filters['company'] !== NULL) {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'name', $queryBuilder->createNamedParameter($filters['company'])
				)
			);
		}

		$companies = $statement->execute()->fetchAll();

		$result = [];
		$result[0] = '';
		foreach ($companies as $company) {
			$result[$company['name']] = $company['name'];
		}

		return $result;
	}

	/**
	 * @param array $filters
	 * @return array
	 * @throws \InvalidArgumentException
	 */
	public static function getAreas(array $filters = []): array {
		$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable(
			'tx_sgjobs_domain_model_company'
		);

		$statement = $queryBuilder->select('a.area')
			->from('tx_sgjobs_domain_model_job', 'a')
			->join(
				'a', 'tx_sgjobs_domain_model_company', 'b'
				, $queryBuilder->expr()->eq('a.company', 'b.uid')
			)
			->where(
				$queryBuilder->expr()->eq(
					'b.country', $queryBuilder->createNamedParameter($filters['country'])
				)
			);

		if ($filters['city'] !== '0' && $filters['city'] !== NULL) {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'b.city', $queryBuilder->createNamedParameter($filters['city'])
				)
			);
		}

		if ($filters['company'] !== '0' && $filters['company'] !== NULL) {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'b.name', $queryBuilder->createNamedParameter($filters['company'])
				)
			);
		}

		$areas = $statement->execute()->fetchAll();

		$result = [];
		$result[0] = '';
		foreach ($areas as $area) {
			$result[$area['area']] = $area['area'];
		}

		return $result;
	}

	/**
	 * @param array $filters
	 * @return array
	 * @throws \InvalidArgumentException
	 */
	public static function getFunctions(array $filters = []): array {
		$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable(
			'tx_sgjobs_domain_model_company'
		);

		$statement = $queryBuilder->select('a.function')
			->from('tx_sgjobs_domain_model_job', 'a')
			->join(
				'a', 'tx_sgjobs_domain_model_company', 'b'
				, $queryBuilder->expr()->eq('a.company', 'b.uid')
			)
			->where(
				$queryBuilder->expr()->eq(
					'b.country', $queryBuilder->createNamedParameter($filters['country'])
				)
			);

		if ($filters['city'] !== '0' && $filters['city'] !== NULL) {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'b.city', $queryBuilder->createNamedParameter($filters['city'])
				)
			);
		}

		if ($filters['company'] !== '0' && $filters['company'] !== NULL) {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'b.name', $queryBuilder->createNamedParameter($filters['company'])
				)
			);
		}

		if ($filters['area'] !== '0' && $filters['area'] !== NULL) {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'b.name', $queryBuilder->createNamedParameter($filters['area'])
				)
			);
		}

		$functions = $statement->execute()->fetchAll();

		$result = [];
		$result[0] = '';
		foreach ($functions as $function) {
			$result[$function['function']] = $function['function'];
		}

		return $result;
	}

	/**
	 * @param array $filters
	 * @param int $siteRootId
	 * @return QueryResult
	 * @throws \InvalidArgumentException
	 */
	public static function getJobs(array $filters = [], $siteRootId): array {
		// get all company ids
		$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable(
			'tx_sgjobs_domain_model_company'
		);

		$statement = $queryBuilder->select('b.uid')
			->from('tx_sgjobs_domain_model_job', 'a')
			->join(
				'a', 'tx_sgjobs_domain_model_company', 'b'
				, $queryBuilder->expr()->eq('a.company', 'b.uid')
			)
			->where(
				$queryBuilder->expr()->eq(
					'b.country', $queryBuilder->createNamedParameter($filters['country'])
				)
			);

		if ($filters['city'] !== '0' && $filters['city'] !== NULL) {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'b.city', $queryBuilder->createNamedParameter($filters['city'])
				)
			);
		}

		if ($filters['company'] !== '0' && $filters['company'] !== NULL) {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'b.name', $queryBuilder->createNamedParameter($filters['company'])
				)
			);
		}

		if ($filters['area'] !== '0' && $filters['area'] !== NULL) {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'b.name', $queryBuilder->createNamedParameter($filters['area'])
				)
			);
		}

		$companies = $statement->execute()->fetchAll();

		$result = [];
		$result[0] = '';
		foreach ($companies as $company) {
			$result[$company['uid']] = $company['uid'];
		}

		// use company ids to filter the jobs
		/** @var JobRepository $jobRepository */
		$jobRepository = GeneralUtility::makeInstance(JobRepository::class);
		return $jobRepository->findByCompanyId($siteRootId, $result);
	}
}
